Google Apps Directory Sync Cookbook
===================================
Installs and configures Google Apps Directory Sync in a Linux environment. In addition, will perform the encryption steps required in the latest versions of Google Apps for Domains Sync.

Requirements
------------
The ``gads::default`` cookbook will install the one dependency gem, ``greenletters`` which is used to automate interaction with the installation script and the password encryption script.

**cookbooks**

* java: Java is required to run Google App Directory Sync. The ``gads::default`` recipe installs using the default behavior. Include java in your runlist prior to gads if you want to change the install behavior.
* run_once: run_once is used to ensure that the installer is only run if ``[:run_once][:gads][:installed]`` is not set or is ``false``.

Attributes
----------

**gads::default**

+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| Attribute                                                       | Type    | Default                                                                                  |
+=================================================================+=========+==========================================================================================+
| ``[:gads][:download_url]``                                      | String  | ``http://dl.google.com/dirsync/dirsync-linux64.sh``                                      |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:create_symlinks]``                                   | Boolean | ``true``                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:install_path]``                                      | String  | ``/opt/GoogleAppsDirSync``                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:synlinks_path]``                                     | String  | ``/usr/local/bin``                                                                       |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:config_path]``                                       | String  | ``/usr/local/etc/gads.xml``                                                              |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:features]``                                          | Array   | * CACHE_PASSWORD_TIMESTAMPS                                                              |
|                                                                 |         | * ALIAS_SYNCHRONIZATION                                                                  |
|                                                                 |         | * USER_PROFILES                                                                          |
|                                                                 |         | * SKIP_CALENDAR_RESOURCES                                                                |
|                                                                 |         | * GROUP_DESCRIPTION                                                                      |
|                                                                 |         | * GROUP_OWNER                                                                            |
|                                                                 |         | * SUSPEND_USERS                                                                          |
|                                                                 |         | * NON_ADDRESS_PRIMARY_KEY                                                                |
|                                                                 |         | * GOOGLE_ORGUNITS                                                                        |
|                                                                 |         | * SHA1_PASSWORD                                                                          |
|                                                                 |         | * MULTIDOMAIN                                                                            |
|                                                                 |         | * SKIP_SUSPENDING_ADMINS                                                                 |
|                                                                 |         | * FAMILY_NAME                                                                            |
|                                                                 |         | * GIVEN_NAME                                                                             |
|                                                                 |         | * SHARED_CONTACTS                                                                        |
|                                                                 |         | * GROUPS                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:logging][:file]``                                    | String  | ``/var/log/google/gads.log``                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:logging][:format]``                                  | String  | ``[%d{ISO8601}] [%t] [%p] [%C{3}] %m%n``                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:logging][:level]``                                   | String  | ``INFO``                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:logging][:max_bytes]``                               | Number  | ``4294967296``                                                                           |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:logging][:max_files]``                               | Number  | ``1``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:smtp_relay]``                         | String  | ``localhost``                                                                            |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:from_address]``                       | String  | ``gads@yourdomain.com``                                                                  |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:to_address]``                         | String  | ``you@yourdomain.com``                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:ignore_errors]``                      | Boolean | ``false``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:ignore_info]``                        | Boolean | ``false``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:notification][:ignore_warnings]``                    | Boolean | ``false``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:delete_calendar_percent]``                   | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:delete_contact_percent]``                    | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:delete_group_percent]``                      | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:delete_ou_percent]``                         | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:delete_user_percent]``                       | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:limit][:suspend_user_percent]``                      | Number  | ``5``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:cache_lifetime]``                                    | Number  | ``691200``                                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:admin_email]``                              | String  | ``you@your-gapps-domain.com``                                                            |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:admin_password]``                           | String  | ``your-password-value``                                                                  |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:domain]``                                   | String  | ``your-gapps-domain.com``                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:threads][:contact_sync]``                   | Number  | ``15``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:threads][:profile_sync]``                   | Number  | ``30``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:threads][:user_sync]``                      | Number  | ``30``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:config_id]``                                         | Unknown | ``Digest::MD5.hexdigest(node[:gads][:google][:domain])``                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:google][:exclude]``                                  | Unknown | ``[]`` - See Usage information below for this data structure                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:type]``                                       | String  | ``OPENLDAP``                                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:connect_method]``                             | String  | ``STANDARD``                                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:hostname]``                                   | String  | ``your-ldap-server``                                                                     |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:port]``                                       | Number  | ``389``                                                                                  |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:basedn]``                                     | String  | ``dc=foo,dc=com``                                                                        |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:auth_type]``                                  | String  | ``SIMPLE``                                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:auth_user]``                                  | String  | ``cn=Manager,dc=foo,dc=com``                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:auth_password]``                              | String  | ``bind dn password value``                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:primary_key]``                                | String  | ``cn``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:email]``                               | String  | ``mail``                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:email_alias]``                         | String  | ``mailLocalAddress``                                                                     |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:given_name]``                          | String  | ``givenName``                                                                            |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:family_name]``                         | String  | ``sn``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:sha1_password]``                       | String  | ``userPassword``                                                                         |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:attr][:password_last_set]``                   | String  | ``pwdChangedTime``                                                                       |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:generated_password_length]``                  | Number  | ``8``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:results_page_size]``                          | Number  | ``1000``                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:priority]``                 | Number  | ``1``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:basedn]``                   | String  | ``ou=Groups,#{node[:gads][:ldap][:basedn]}``                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:scope]``                    | String  | ``SUBTREE``                                                                              |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:filter]``                   | String  | ``objectClass=groupOfNames``                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:attr][:description]``       | String  | ``description``                                                                          |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:attr][:display_name]``      | String  | ``cn``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:attr][:member]``            | String  | ``member``                                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:attr][:name]``              | String  | ``mailRoutingAddress``                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:groups][:search][:attr][:owner_dn]``          | String  | ``owner``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:mapping][:dn]``                    | String  | ``ou=People,#{node[:gads][:ldap][:basedn]}``                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:mapping][:name]``                  | String  | ``/``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:search][:priority]``               | Number  | ``1``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:search][:basedn]``                 | String  | ``ou=People,#{node[:gads][:ldap][:basedn]}``                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:search][:scope]``                  | String  | ``SUBTREE``                                                                              |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:search][:filter]``                 | String  | ``objectClass=organizationalUnit``                                                       |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:orgunits][:search][:attr][:description]``     | String  | ``ou``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:priority]``                  | Number  | ``1``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:basedn]``                    | String  | ``ou=People,#{node[:gads][:ldap][:basedn]}``                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:suspended]``                 | Boolean | ``false``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:scope]``                     | String  | ``SUBTREE``                                                                              |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:filter]``                    | String  | ``objectClass=organizationalUnit``                                                       |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:org_mapping]``        | String  |                                                                                          |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:company_name]``       | Number  | ``0``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:department]``         | String  | ``department``                                                                           |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:title]``              | String  | ``title``                                                                                |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:full_name]``          | String  | ``displayName``                                                                          |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:assistant]``          | String  | ``secretary``                                                                            |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:manager]``            | String  | ``manager``                                                                              |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:sync_key]``           | String  | ``cn``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:street_address]``     | String  | ``street``                                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:pobox]``              | String  | ``postOfficeBox``                                                                        |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:city]``               | String  | ``l``                                                                                    |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:state]``              | String  | ``st``                                                                                   |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:postal_code]``        | String  | ``postalCode``                                                                           |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:email]``              | String  | ``mail``                                                                                 |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:employee_id]``        | String  | ``employeeNumber``                                                                       |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:fax]``                | String  | ``faxsimileTelephoneNumber``                                                             |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:home_phone]``         | String  | ``homePhone``                                                                            |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:mobile_phone]``       | String  | ``mobile``                                                                               |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:work_phone]``         | String  | ``telephoneNumber``                                                                      |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+
| ``[:gads][:ldap][:users][:search][:attr][:description]``        | String  | ``description``                                                                          |
+-----------------------------------------------------------------+---------+------------------------------------------------------------------------------------------+

Usage
-----
**gads::default**

Just include gads and java in your node's ``run_list``:

.. code-block:: python

    {
      "name": "my_node",
      "run_list": [
        "recipe[java]"
        "recipe[gads]"
      ]
    }

**Managing the exclusion list**

The exclusion list is managed with the ``[:gads][:google][:exclude]`` Array. In the array, is a :Hash with three keys: ``:match``, ``:type``, and ``:filter``. For an explanation of the exclude behavior and types, see the Google Apps for Domains documentation.

Example:

.. code-block:: python

   [{:match => 'USER_NAME',
     :type =>  'EXACT',
     :filter => 'foo@bar.com'},
    {:match => 'USER_NAME',
     :type => 'SUBSTRING',
     :filter => 'txt.att.net'},
    {:match => 'GROUP_NAME',
     :type => 'EXACT',
     :filter => 'gapps-only-group@your-gapps-domain.com'}]

**Run control**

Installation will toggle node attributes that control subsequent runs:

+---------------------------------------------+---------+---------------------------------------------------------------------------------------------+-----------+
| Key                                         | Type    | Description                                                                                 | Value *   |
+=============================================+=========+=============================================================================================+===========+
| ``[:run_once][:gads][:installed]``          | Boolean | If true, will prevent gads from being re-downloaded and re-installed.                       | ``true``  |
+---------------------------------------------+---------+---------------------------------------------------------------------------------------------+-----------+

- If you want to install a GADS version update, you will need to set the ``[:run_once][:gads][:installed]`` value to ``false`` to download and install the new version.

License and Authors
-------------------
Author:: Gavin M. Roy (gmr@meetme.com) Copyright:: 2013, MeetMe, Inc

Copyright (c) 2013, MeetMe, Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the MeetMe, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
